/*
	Genome-wide Efficient Mixed Model Association (GEMMA)
    Copyright (C) 2011  Xiang Zhou and Matthew Stephens

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __IO_H__                
#define __IO_H__


#include <vector>
#include <map>
#include "gsl/gsl_vector.h"
#include "gsl/gsl_matrix.h"

using namespace std;





class SNPINFO {
public:
  SNPINFO();
  SNPINFO(const int &i, const string &r, const int &c, const long int &j, const char &a, const char &b): chr(i), rs_number(r), cM(c), base_position(j), a_major(a), a_minor(b) { }	

  int get_chr() const {return chr;}
  string get_rs() const {return rs_number;}
  int get_cM() const {return cM;}
  long int get_pos() const {return base_position;}
  char get_major() const {return a_major;}
  char get_minor() const {return a_minor;} 

private:
  int chr;
  string rs_number;
  int cM;
  long int base_position;
  char a_major;
  char a_minor;
};




class SUMSTAT {
public:
  int chr;
  string rs_number;
  long int base_position;
  int n_miss;				//number of missing genotypes
  double beta;				//REML estimator for beta
  double se;				//SE for beta  
  double lambda_remle;		//REML estimator for lambda
  double lambda_mle;		//MLE estimator for lambda
  double p_wald;			//p value from a Wald test
  double p_lrt;				//p value from a likelihood ratio test
  double p_score;			//p value from a score test
};




void ProgressBar (string str, double p, double total);
bool ReadFile_bim (const string &file_bim, vector<SNPINFO> &snp_info);
bool ReadFile_fam (const string &file_fam, vector<int> &indicator_idv, vector<double> &pheno, map<string, int> &mapID2num, const int &p_column);

bool ReadFile_cvt (const string &file_cvt, vector<int> &indicator_cvt, vector<vector<double> > &cvt, size_t &n_cvt);
bool ReadFile_anno (const string &file_bim, map<string, int> &mapRS2chr, map<string, long int> &mapRS2bp);
bool ReadFile_pheno (const string &file_fam, vector<int> &seq_i, vector<double> &pheno, const int &p_ordinal);

bool ReadFile_geno (const string &file_geno, vector<int> &indicator_idv, vector<int> &indicator_snp, const double &maf_level, const double &miss_level, map<string, int> &mapRS2chr, map<string, long int> &mapRS2bp, vector<SNPINFO> &snpInfo, size_t &ns_test);
bool ReadFile_bed (const string &file_bed, vector<int> &indicator_idv, vector<int> &indicator_snp, const double &maf_level, const double &miss_level, const size_t &ns_total, size_t &ns_test);

bool BimbamKin (const string &file_geno, vector<int> &indicator_snp, const int k_mode, const int display_pace, gsl_matrix *matrix_kin);
bool PlinkKin (const string &file_bed, vector<int> &indicator_snp, const int k_mode, const int display_pace, gsl_matrix *matrix_kin);

#endif







